<?php
/**
 * Class to build and display bars charts
 * @author José Carlos Cruz Parra AKA internia
 * josecarlos@programadorphpfreelance.com
 * http://programadorphpfreelance.com
 * 2008-2009
 * This code is released under the GNU General Public License.
 */
class MyChart
{
	var $pixels100 = 300;
	var $items = array();
	var $hBarHeight = 10;
	var $barColor = "#2a47b5";
	var $titulo = "";
	var $claseListadoClaro = "listadoclaro";
	var $claseListadoNormal = "listadonormal";
	var $_mostrarTotal = false;

	function MyChart($titulo="")
	{
		$this->titulo = $titulo;
	}
	/**
	 * Añade un item
	 *
	 * @param MyItemChart $item
	 */
	function AddItem($item)
	{
		$this->items[] = $item;
	}
	function BuildHorizontal($conPorcentajes=false)
	{
		$suma = $this->getSuma();
		$multiplicador = $this->getMultiplicador();
		$rowHeight = $this->hBarHeight + 6;
		$chart = "<div style=\"text-align:center;padding:4px\">"
			."<table border=\"0\" cellpadding=\"0\" cellspacing=\"0\" style=\"margin:0 auto;text-align:left\">"
			."<tr><td style=\"text-align:center;padding:4px\" colspan=\"3\"><b>{$this->titulo}</b></td></tr>";
		for($c=0,$size=sizeof($this->items); $c<$size; $c++)
		{
			$clase = ($c%2 == 0) ? $this->claseListadoClaro : $this->claseListadoNormal;
			$pctj = (100 * $this->items[$c]->cantidad) / $suma;
			$pixels = round(($pctj * $this->pixels100) / 100);
			$pixels = $pixels * $multiplicador;
			$chart .= sprintf("<tr style=\"height:%spx\">"
				."<td style=\"text-align:right\">%s&nbsp;</td>"
				."<td class=\"%s\" style=\"vertical-align:middle\">"
					."<div style=\"float:left;width:%spx;height:%spx;background:%s\">",
				$rowHeight, $this->items[$c]->label, $clase, $pixels, $this->hBarHeight, $this->barColor);
			if(sizeof($this->items[$c]->subs) > 0)
			{
				$aux = "";
				foreach($this->items[$c]->subs as $sub)
				{
					$subpixels = round($sub * $pixels / $this->items[$c]->cantidad);
					$chart .= sprintf("<div style=\"float:left;width:%spx;height:%spx;background:#ff0000\"></div>", $subpixels, $this->hBarHeight);
				}
			}
			$chart .= sprintf("</div>%s"
				."</td>"
				."</tr>",
				$conPorcentajes ? $this->getPorcentaje($suma, $this->items[$c]->cantidad) : $this->items[$c]->cantidad);
		}
		$chart .= "</table></div>";
		return $chart;
	}
	function BuildVertical($conPorcentajes=false)
	{
		$suma = $this->getSuma();
		$multiplicador = $this->getMultiplicador();
		$rowHeight = $this->hBarHeight + 6;
		$total = 0;
		$chart = sprintf("<div style=\"text-align:center;padding:4px\">"
			."<table border=\"0\" cellpadding=\"0\" cellspacing=\"0\" style=\"margin:0 auto;text-align:left\">"
			."<tr><td style=\"text-align:center;padding:4px\" colspan=\"%s\"><b>%s</b></td></tr>"
			."<tr style=\"height:%spx\">", sizeof($this->items), $this->titulo, $this->pixels100);
		for($c=0,$size=sizeof($this->items); $c<$size; $c++)
		{
			$total += $this->items[$c]->cantidad;
			$clase = ($c%2 == 0) ? $this->claseListadoClaro : $this->claseListadoNormal;
			$pctj = $suma == 0 ? 0 : (100 * $this->items[$c]->cantidad) / $suma;
			$pixels = round(($pctj * $this->pixels100) / 100);
			$pixels = $pixels * $multiplicador;
			$chart .= sprintf(
				"<td class=\"%s\" style=\"vertical-align:bottom;text-align:center;width:%spx\">"
					."%s",
				$clase, $this->hBarHeight,	$conPorcentajes ? $this->getPorcentaje($suma, $this->items[$c]->cantidad) : $this->items[$c]->cantidad);
			if($this->items[$c]->cantidad > 0)
			{
				$chart .= sprintf("<div style=\"width:%spx;height:%spx;background:%s\">",
					$this->hBarHeight, $pixels,	$this->barColor);

				if(sizeof($this->items[$c]->subs) > 0)
				{
					$aux = "";
					$restopixels = $pixels;
					foreach($this->items[$c]->subs as $sub)
					{
						$subpixels = round($sub * $pixels / $this->items[$c]->cantidad);
						if($subpixels > 0)
						{
							$aux .= sprintf("<div style=\"height:%spx;background:#ff0000\"></div>", $subpixels);
							$restopixels -= $subpixels;
						}
					}
					if($restopixels < $pixels && $restopixels > 0)
					{
						$chart .= sprintf("<div style=\"height:%spx\"></div>", $restopixels);
					}
					$chart .= $aux;
				}
				$chart .= "</div>";
			}
			$chart .= "</td>";
		}
		$chart .= "</tr><tr>";
		for($c=0,$size=sizeof($this->items); $c<$size; $c++)
		{
			$chart .= sprintf(
				"<td style=\"text-align:center\">%s</td>",
				$this->items[$c]->label);
		}
		$chart .= "</tr>";
		$chart .= "</table>";

		if($this->_mostrarTotal)
		{
			$chart .= "<div style=\"padding-top:10px;text-align:left\">Total: {$total}</div>";
		}

		return $chart."</div>";
	}
	/**
	 * Obtiene la suma de las cantidades de los items
	 * private
	 *
	 * @return double
	 */
	function getSuma()
	{
		$suma = 0;
		for($c=0,$size=sizeof($this->items); $c<$size; $c++)
		{
			$suma += $this->items[$c]->cantidad;
		}
		return $suma;
	}
	/**
	 * Obtiene el porcentaje de una cantidad (de un item) respecto a la suma (de todas las cantidades)
	 *
	 * @param double $suma
	 * @param double $cantidad
	 * @return float
	 */
	function getPorcentaje($suma, $cantidad)
	{
		return sprintf("%.2f%%", 100 / ($suma / $cantidad) );
	}
	/**
	 * Obtiene el número por el que multiplicar los pixels correspondientes a los items para ajustar el gráfico a la mayor de las cantidades
	 * private
	 *
	 * @return double
	 */
	function getMultiplicador()
	{
		$cantidad = 0;
		for($c=0,$size=sizeof($this->items); $c<$size; $c++)
		{
			if($this->items[$c]->cantidad > $cantidad)
			{
				$cantidad = $this->items[$c]->cantidad;
			}
		}
		$suma = $this->getSuma();
		return $suma == 0 ? 0 : round(100 / ((100 * $cantidad) / $this->getSuma()));
	}
	/**
	 * Establece el título del gráfico
	 *
	 * @param string $tilulo
	 */
	function SetTitle($tilulo)
	{
		$this->titulo = $tilulo;
	}
	/**
	 * Establece el color de las barras
	 *
	 * @param string $color
	 */
	function SetBarColor($color)
	{
		$this->barColor = $color;
	}
	/**
	 * Establece la clase CSS para las filas claras
	 *
	 * @param string $clase
	 */
	function SetClaseListadoClaro($clase)
	{
		$this->claseListadoClaro = $clase;
	}
	/**
	 * Establece la clase CSS para las filas oscuritas
	 *
	 * @param string $clase
	 */
	function SetClaseListadoNormal($clase)
	{
		$this->claseListadoNormal = $clase;
	}
	/**
	 * Establece el número de pixels correspondiente a un 100%
	 *
	 * @param int $pixels
	 */
	function SetPixels100($pixels)
	{
		$this->pixels100 = $pixels;
	}
	/**
	 * Establece la altura en pixels de las barras
	 *
	 * @param unknown_type $height
	 */
	function SetHBarHeight($height)
	{
		$this->hBarHeight = $height;
	}
	/**
	 * Establece si mostrar el total o no
	 *
	 * @param bool $valor
	 */
	function SetMostrarTotal($valor)
	{
		$this->_mostrarTotal = $valor ? true : false;
	}
	/**
	 * Limpia el conjunto de items
	 *
	 */
	function ClearItems()
	{
		$this->items = array();
	}
}
/**
 * Clase item de un gráfico
 *
 */
class MyItemChart
{
	var $label = "";
	var $cantidad = 0;

	var $subs = array();

	/**
	 * Constructor
	 *
	 * @param string $label
	 * @param double $cantidad
	 * @param mixed $subs
	 * @return MyItemChart
	 */
	function MyItemChart($label, $cantidad, $subs=array())
	{
		$this->label = $label;
		$this->cantidad = $cantidad;
		if(is_array($subs))
		{
			$this->subs = $subs;
		}
		else
		{
			$this->subs[] = $subs;
		}
	}
	function GetSubsHTML()
	{
		$html = "";
		foreach($this->subs as $sub)
		{
			$html .= sprintf("<div style=\"width:%spx;height:%spx;background:%s\"></div>");
		}
		return $html;
	}
}
/**
 * Class to build and display horizontal bars charts
 * @author José Carlos Cruz Parra AKA internia
 * josecarlos@programadorphpfreelance.com
 * http://programadorphpfreelance.com
 * 2008
 * This code is released under the GNU General Public License.
 *
 * SE MANTIENE POR COMPATIBILIDAD CON ANTERIORES VERSIONES
 */
class MyChart_Horizontal
{
	var $pixels100 = 300;
	var $items = array();
	var $hBarHeight = 10;
	var $barColor = "#2a47b5";
	var $titulo = "";
	var $claseListadoClaro = "listadoclaro";
	var $claseListadoNormal = "listadonormal";

	function MyChart_Horizontal($titulo="")
	{
		$this->titulo = $titulo;
	}
	/**
	 * Añade un item
	 *
	 * @param MyItemChart $item
	 */
	function AddItem($item)
	{
		$this->items[] = $item;
	}
	function Build($conPorcentajes=false)
	{
		$suma = $this->getSuma();
		$multiplicador = $this->getMultiplicador();
		$rowHeight = $this->hBarHeight + 6;
		$chart = "<div style=\"text-align:center;padding:4px\">"
			."<table border=\"0\" cellpadding=\"0\" cellspacing=\"0\" style=\"margin:0 auto;text-align:left\">"
			."<tr><td style=\"text-align:center;padding:4px\" colspan=\"3\"><b>{$this->titulo}</b></td></tr>";
		for($c=0,$size=sizeof($this->items); $c<$size; $c++)
		{
			$clase = ($c%2 == 0) ? $this->claseListadoClaro : $this->claseListadoNormal;
			$pctj = (100 * $this->items[$c]->cantidad) / $suma;
			$pixels = round(($pctj * $this->pixels100) / 100);
			$pixels = $pixels * $multiplicador;
			$chart .= sprintf("<tr style=\"height:%spx\">"
				."<td style=\"text-align:right\">%s&nbsp;</td>"
				."<td class=\"%s\" style=\"vertical-align:middle\">"
					."<div style=\"float:left;width:%spx;height:%spx;background:%s\"></div>%s"
				."</td>"
				."</tr>",
				$rowHeight, $this->items[$c]->label, $clase, $pixels, $this->hBarHeight, $this->barColor, $conPorcentajes ? $this->getPorcentaje($suma, $this->items[$c]->cantidad) : $this->items[$c]->cantidad);
		}
		$chart .= "</table></div>";
		return $chart;
	}
	/**
	 * Obtiene la suma de las cantidades de los items
	 * private
	 *
	 * @return double
	 */
	function getSuma()
	{
		$suma = 0;
		for($c=0,$size=sizeof($this->items); $c<$size; $c++)
		{
			$suma += $this->items[$c]->cantidad;
		}
		return $suma;
	}
	/**
	 * Obtiene el porcentaje de una cantidad (de un item) respecto a la suma (de todas las cantidades)
	 *
	 * @param double $suma
	 * @param double $cantidad
	 * @return float
	 */
	function getPorcentaje($suma, $cantidad)
	{
		return sprintf("%.2f%%", 100 / ($suma / $cantidad) );
	}
	/**
	 * Obtiene el número por el que multiplicar los pixels correspondientes a los items para ajustar el gráfico a la mayor de las cantidades
	 * private
	 *
	 * @return double
	 */
	function getMultiplicador()
	{
		$cantidad = 0;
		for($c=0,$size=sizeof($this->items); $c<$size; $c++)
		{
			if($this->items[$c]->cantidad > $cantidad)
			{
				$cantidad = $this->items[$c]->cantidad;
			}
		}
		return round(100 / ((100 * $cantidad) / $this->getSuma()));
	}
	/**
	 * Establece el título del gráfico
	 *
	 * @param string $tilulo
	 */
	function SetTitle($tilulo)
	{
		$this->titulo = $tilulo;
	}
	/**
	 * Establece el color de las barras
	 *
	 * @param string $color
	 */
	function SetBarColor($color)
	{
		$this->barColor = $color;
	}
	/**
	 * Establece la clase CSS para las filas claras
	 *
	 * @param string $clase
	 */
	function SetClaseListadoClaro($clase)
	{
		$this->claseListadoClaro = $clase;
	}
	/**
	 * Establece la clase CSS para las filas oscuritas
	 *
	 * @param string $clase
	 */
	function SetClaseListadoNormal($clase)
	{
		$this->claseListadoNormal = $clase;
	}
	/**
	 * Establece el número de pixels correspondiente a un 100%
	 *
	 * @param int $pixels
	 */
	function SetPixels100($pixels)
	{
		$this->pixels100 = $pixels;
	}
	/**
	 * Establece la altura en pixels de las barras
	 *
	 * @param unknown_type $height
	 */
	function SetHBarHeight($height)
	{
		$this->hBarHeight = $height;
	}
}
?>