<?php
/**
 * Clase con funciones estáticas genéricas para instalar SWPHP
 *
 * Jose Carlos Cruz Parra
 * programadorphpfreelance.com
 * 2012-08-14
 *
 * SIMPLE WEB PHP EDICION ADMINISTRADA
 *
 * install/inc/classes/SWPHP_Install.class.php
 *
 */
class SWPHP_Install
{
    const APP_NAME = 'Kptl Truth';
    const APP_VERSION = '2.0.1';
    const APP_WEBSITE = 'http://www.kapitaltruth.org/';
    const CONFIG_FILE = 'config/config-SWPHP.inc.php';
    private static $configConstArr = array(
        'SWPHP_DBHOST'=>'localhost', //Servidor de la base de datos
        'SWPHP_DBPORT'=>3306, //Puerto para conectar con la base de datos
        'SWPHP_DBUSER'=>'root', //Usuario para la base de datos
        'SWPHP_DBPASS'=>'root', //Contraseña para la base de datos
        'SWPHP_DBNAME'=>'swphp_ea', //Nombre de la base de datos
        'SWPHP_DBPREFIX'=>'swphp_', //Prefijo de las tablas

        'SWPHP_DEBUG'=>false, //Modo depuración sí o no
        'SWPHP_COMMON_DIR'=>'inc/common.swphp/', //Ruta desde el raiz de la web al directorio donde se encuentran recursos y librerías comunes (smarty, phpmailer, classes, functions,...)
        'SWPHP_ADMIN_DIR'=>'admin/', //Directorio del panel de administración, terminado en '/'
        'SWPHP_INC_DIR'=>'inc/', //Ruta desde el raiz de la web al directorio donde se encuentran los módulos, plugins, y otros elementos comunes propios de SWPHP EA
        'SWPHP_THEMES_DIR'=>'themes/', //Ruta desde el raiz de la web al directorio donde se encuentran los themes (temas gráficos)
        'SWPHP_TEMP_DIR'=>'temp/', //Directorio por defecto para archivos temporales, terminado en '/'
        'SWPHP_UPLOADS_DIR'=>'files/uploads/', //Directorio donde guardar los archivos subidos, terminado en '/'
        'SWPHP_MAX_UPLOAD_FILE_SIZE'=>20000, //Tamaño máximo para subir un archivo, en Kbs
        'SWPHP_ALLOWED_UPLOAD_FILE_EXTENSIONS'=>'bmp,gif,jpg,jpeg,png,tiff,ico,psd,swf,txt,xml,doc,docx,mdb,xls,xlt,pps,ppt,pdf,xsn,odt,ods,odb,csv,rtf,mp3,wma,wmv,avi,mpg,mid,bmw,bmx,flv,mpeg,mov,mp4,wav,rar,zip,ace', //Extensiones permitidas para subir archivos, separadas por coma
        'SWPHP_DEFAULT_ICON'=>'logo.png', //Icono por defecto para páginas, módulos,...
        'SWPHP_DEFAULT_ICON_WIDTH'=>48, //Ancho por defecto para iconos de páginas, módulos,... Valores válidos: 16,24,32,48,64,128
        'SWPHP_MAX_GET_PAGES'=>9, //Profundidad máxima del árbol de páginas
        'SWPHP_FRASE_SECRETA'=>'agdDyhh+76dgssaa7', //Cadena para generar hashes
        'SWPHP_SESSION_EXPIRE_TIME'=>3600, //Tiempo de caducidad de la sesión, en segundos
        'SWPHP_PASSWORD_MINLENGTH'=>6, //Longitud mínima para contraseñas
        'SWPHP_PASSWORD_MAXLENGTH'=>20, //Longitud máxima para contraseñas
        'SWPHP_PASSWORD_VALIDCHARS'=>'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789.;,_-+ *@#$%&/()=\'<>^~ñÑ', //Caracteres válidos para contraseñas
        'SWPHP_NICK_MINLENGTH'=>3, //Longitud mínima para alias de usuario
        'SWPHP_NICK_MAXLENGTH'=>32, //Longitud máxima para alias de usuario
        'SWPHP_NICK_VALIDCHARS'=>'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789.;,_-+ *@#$%&/()=\'<>^~ñÑ', //Caracteres válidos para alias de usuario
        );
    /**
     * Obtiene un array asociativo con las constantes de configuración y sus valores actualmente asignados
     *
     * @return array
     */
    public static function GetConfigConstArr()
    {
        foreach(SWPHP_Install::$configConstArr as $key=>$value)
        {
            if(defined($key))
            {
                SWPHP_Install::$configConstArr[$key] = constant($key);
            }
        }

        return SWPHP_Install::$configConstArr;
    }
    /**
     * Define las constantes de configuración
     */
    public static function DefineConfigConst()
    {
        $arr = SWPHP_Install::GetConfigConstArr();

        foreach($arr as $key=>$value)
        {
            if(!defined($key))
            {
                define($key, $value);
            }
        }
    }
    /**
     * Comprueba los requisitos de instalación
     *
     * @return boolean
     */
    public static function CheckEnvironment()
    {
        $ok = true;

        //Servidor web

        if(stripos($_SERVER['SERVER_SOFTWARE'], 'apache') !== false)
        {
            Msg(SWPHP_Install::l('Servidor Apache'), 'check');
        }
        else
        {
            Msg(SWPHP_Install::l('Servidor:').' '.$_SERVER['SERVER_SOFTWARE'], 'aviso');
            Msg(SWPHP_Install::l('Se recomienda servidor Apache'), 'info');
        }

        //Versión de PHP

        $val = phpversion();
        if($val < '5.0.0')
        {
            Msg(SWPHP_Install::l('Esta versión de PHP no es soportada:')." {$val}", 'error');
            $ok = false;
        }
        else
        {
            Msg(SWPHP_Install::l('Versión de PHP soportada'), 'check');
        }

        //Register globals

        $val = ini_get('register_globals');
        if($val)
        {
            Msg(SWPHP_Install::l('Directiva register_globals habilitada'), 'aviso');
            Msg(SWPHP_Install::l('Es recomendable trabajar con register_globals deshabilitada'), 'info');
        }
        else
        {
            Msg(SWPHP_Install::l('Directiva register_globals deshabilitada'), 'check');
        }

        //Magic quotes

        $val = get_magic_quotes_gpc();
        if($val)
        {
            Msg(SWPHP_Install::l('Directiva magic_quotes habilitada'), 'aviso');
            Msg(SWPHP_Install::l('Es recomendable trabajar con magic_quotes deshabilitada'), 'info');
        }
        else
        {
            Msg(SWPHP_Install::l('Directiva magic_quotes deshabilitada'), 'check');
        }

        //Max execution time

        $val = ini_get('max_execution_time');
        if($val < 60)
        {
            @set_time_limit(4321);
            $val = ini_get('max_execution_time');

            if($val < 60)
            {
                Msg(SWPHP_Install::l('El tiempo máximo de ejecución de PHP establecido es').' '.$val.' '.SWPHP_Install::l('segundos'), 'aviso');
                Msg(SWPHP_Install::l('Es recomendable trabajar con un tiempo máximo de ejecución de al menos 60 segundos'), 'info');
            }
            else
            {
                Msg(SWPHP_Install::l('Tiempo máximo de ejecución de PHP comprobado'), 'check');
            }
        }
        else
        {
            Msg(SWPHP_Install::l('Tiempo máximo de ejecución de PHP comprobado'), 'check');
        }

        //Archivo de configuración

        if(file_exists(SWPHP_Install::CONFIG_FILE))
        {
            Msg(SWPHP_Install::l('Existe el archivo de configuración'), 'check');
            if(is_writable(dirname(SWPHP_Install::CONFIG_FILE)) && is_writable(SWPHP_Install::CONFIG_FILE))
            {
                Msg(SWPHP_Install::l('Se puede escribir en el archivo de configuración'), 'check');
            }
            else
            {
                Msg(SWPHP_Install::l('No se puede escribir en el archivo de configuración'), 'error');
                Msg(SWPHP_Install::l('Comprueba los permisos sobre el archivo')." <b>".SWPHP_Install::CONFIG_FILE."</b> ".SWPHP_Install::l('y el directorio')." <b>".dirname(SWPHP_Install::CONFIG_FILE)."</b>", 'info');
                $ok = false;
            }
        }
        else
        {
            Msg(SWPHP_Install::l('No se encuentra el archivo de configuración'), 'error');
            Msg(SWPHP_Install::l('Asegúrate de subir todos los archivos incluidos en').' '.SWPHP_Install::APP_NAME, 'info');
            $ok = false;
        }

        //Directorio de plantillas compiladas

        if(SWPHP_Install::CheckDirWritable('templates_c'))
        {
            Msg(SWPHP_Install::l('Se puede escribir en el directorio de plantillas compiladas'), 'check');
        }
        else
        {
            $ok = false;
        }

        //Directorio de archivos temporales

        if(SWPHP_Install::CheckDirWritable('temp'))
        {
            Msg(SWPHP_Install::l('Se puede escribir en el directorio de archivos temporales'), 'check');
        }
        else
        {
            $ok = false;
        }

        //Directorio de archivos de registro

        if(SWPHP_Install::CheckDirWritable('files/log'))
        {
            Msg(SWPHP_Install::l('Se puede escribir en el directorio de archivos de registro'), 'check');
        }
        else
        {
            $ok = false;
        }

        return $ok;
    }
    /**
	 * Comprueba si existe y se puede escribir sobre un directorio
	 *
	 * @param string $dir
	 * @param bool $createIfNotExists
	 * @param bool $dieOnFail
	 * @param bool $level
	 * @return bool
	 */
	public static function CheckDirWritable($dir, $createIfNotExists=true, $dieOnFail=false, $level=0)
	{
		$dirs = explode('/', $dir);
		$dirs = array_filter($dirs);
		$size = sizeof($dirs);
		if($level == 0 && $size > 1)
		{
			$return = true;
			$path = '';
			for($c=0; $c<$size; $c++)
			{
				$path .= "{$dirs[$c]}/";
				if($c >= $level)
				{
					$return = SWPHP_Install::CheckDirWritable($path, $createIfNotExists, $dieOnFail, $level+1) && $return;
				}
			}

			return $return;
		}
		else
		{
			$path = $dir;

			if(file_exists($path))
			{
				if(is_writable($path))
				{
					return true;
				}
				elseif($level == 0)
				{
					$msg = SWPHP_Install::l('Se necesitan permisos de escritura sobre el directorio')." <b>{$path}</b>";
					if($dieOnFail)
					{
						die($msg);
					}
					else
					{
						Msg($msg, 'error');
					}
				}
			}
			elseif($createIfNotExists)
			{
				SWPHP::_MkDir($path, true, 0775);

				return SWPHP_Install::CheckDirWritable($path, false, $dieOnFail);
			}
			else
			{
				$msg = SWPHP_Install::l('No se encuentra el directorio')." <b>{$path}</b>";
				if($dieOnFail)
				{
					die($msg);
				}
				else
				{
					Msg($msg, 'error');
				}
			}
		}

		return false;
	}
    /**
     * Asigna valores a las constantes de configuración (sin llegar a guardar)
     *
     * @param array $data
     */
    public static function SetConfigConst($data=array())
    {
        foreach($data as $key=>$value)
        {
            if(isset(SWPHP_Install::$configConstArr[$key]))
            {
                SWPHP_Install::$configConstArr[$key] = $value;
            }
            elseif(isset(SWPHP_Install::$configConstArr['SWPHP_'.strtoupper($key)]))
            {
                SWPHP_Install::$configConstArr['SWPHP_'.strtoupper($key)] = $value;
            }
        }
    }
    /**
     * Escribe la configuración en el archivo de configuración
     *
     * @return boolean
     */
    public static function SaveConfigConst()
    {
        $fp = fopen(SWPHP_Install::CONFIG_FILE, 'w');

        if($fp)
        {
            fwrite($fp, "<?php\n");

            foreach(SWPHP_Install::$configConstArr as $key=>$value)
            {
                fwrite($fp, sprintf("define('%s', %s);\n", $key, SWPHP_Install::value2define($value)));
            }

            fwrite($fp, "?>");

            fclose($fp);

            return true;
        }

        return false;
    }
    /**
     * Prepara un valor para escribirlo en un define en el archivo de configuración
     *
     * @param mixed $val
     * @return string
     */
    private static function value2define($val)
    {
        if(is_bool($val))
        {
            return $val ? 'true' : 'false';
        }
        elseif(is_numeric($val))
        {
            return number_format($val, 6, '.', '');
        }
        else//if(is_string($val))
        {
            return sprintf("'%s'", addcslashes($val, "'"));
        }
    }
    /**
     * Establece valores de configuración en la base de datos
     *
     * @param array $data
     * @return bool
     */
    public static function SaveConfigValues($data)
    {
        $ok = true;

        $tablas = array(
            'config'=>array(
                'configuracion'=>array(
                    'organizacion',
                    'dominio',
                    'path',
                    ),
                'configuracion_email'=>array(
                    'emailenvios',
                    'nombreenvios',
                    'usar_smtp',
                    'smtp_user',
                    'smtp_pass',
                    'smtp_host',
                    'smtp_port',
                    ),
                ),
            'kptl_config'=>array(
                'kapitaltruth'=>array(
                    'KPTL_punto',
                    'KPTL_puntos',
                    'potencia',
                    'tesorototal',
                    'tesoroactual',
                    ),
                ),
            );

        foreach($tablas as $tabla=>$conceptos)
        {
            foreach($conceptos as $concepto=>$vars)
            {
                foreach($vars as $var)
                {
                    if(isset($data[$var]))
                    {
                        if(!SWPHP::_SetConfigValue($var, $data[$var], $concepto, true, $tabla))
                        {
                            $ok = false;

                            Msg(SWPHP_Install::l('No se ha podido actualizar el valor de configuración')." <b>{$var}</b>", 'error');
                        }
                    }
                    elseif(isset($data[$tabla][$var]))
                    {
                        if(!SWPHP::_SetConfigValue($var, $data[$tabla][$var], $concepto, true, $tabla))
                        {
                            $ok = false;

                            Msg(SWPHP_Install::l('No se ha podido actualizar el valor de configuración')." <b>{$var}</b>", 'error');
                        }
                    }
                    else
                    {
                        $ok = false;

                        Msg(SWPHP_Install::l('Falta el valor de configuración')." <b>{$var}</b>", 'error');
                    }
                }
            }
        }

        return $ok;
    }
    /**
     * Prepara los datos de configuración básica
     *
     * @param array $data
     * @return array
     */
    public static function PrepararBasica($data)
    {
        if(substr($data['dominio'], 0, 7) == 'http://')
        {
            $data['dominio'] = substr($data['dominio'], 7);
        }
        elseif(substr($data['dominio'], 0, 8) == 'https://')
        {
            $data['dominio'] = substr($data['dominio'], 8);
        }
        /*
        if(substr($data['dominio'], 0, 4) == 'www.')
        {
            $data['dominio'] = substr($data['dominio'], 4);
        }
        */

        $data['path'] = ponerBarra($data['path']);

        if(strlen($data['path']) > 1 && substr($data['path'], 0, 1) == '/')
        {
            $data['path'] = substr($data['path'], 1);
        }

        //$data['url'] = ponerBarra($data['url']);

        $data['admin_dir'] = ponerBarra($data['admin_dir']);

        $data['kptl_config']['tesorototal'] = pow(10, $data['kptl_config']['potencia']);
        $data['kptl_config']['tesoroactual'] = $data['kptl_config']['tesorototal'];

        return $data;
    }
    /**
     * Obtiene $db
     *
     * @return \DB_Connection|boolean
     */
    public static function GetDb()
    {
        global $db;

        if(isset($db) && is_object($db))
        {
            return $db;
        }
        else
        {
            $db = DB_Connection::Factory(SWPHP_DBHOST, SWPHP_DBPORT, SWPHP_DBNAME, SWPHP_DBUSER, SWPHP_DBPASS, false);

            if($db->Connect(false) != false)
            {
                if($db->SelectDB() != false)
                {
                    return $db;
                }
                else
                {
                    Msg(SWPHP_Install::l('No se ha podido seleccionar la base de datos')." <b>".SWPHP_DBNAME."</b><br />".$db->Error(), 'error');
                    Msg(SWPHP_Install::l('Comprueba que la base de datos existe y el usuario tiene permisos sobre ella.'), 'info');
                }
            }
            else
            {
                Msg(SWPHP_Install::l('No se ha podido conectar con el servidor de base de datos')."<br />".$db->Error(), 'error');
                Msg(SWPHP_Install::l('Comprueba los datos de conexión'), 'info');
            }
        }

        return false;
    }
    /**
     * Crea un usuario
     *
     * @param array $data
     * @param int $perfil_id
     * @return boolean
     */
    public static function CrearUser($data, $perfil_id)
    {
        $db = SWPHP_Install::GetDb();

        $ok = true;

        $ahora = date('Y-m-d H:i:s');

        $user = array(
            'nick'=>$data['nick'],
            'pass'=>SWPHP::_Encrypt($data['pass']),
            'email'=>$data['email'],
            'perfil_id'=>$perfil_id,
            'fechayhoraregistro'=>$ahora,
            'fechayhoraultima'=>$ahora,
            'activo'=>1,
            'contadorlogin'=>0,
            'ultimologin'=>'0000-00-00 00:00:00',
            'user_idregistro'=>0,
            'user_idultima'=>0,
            );
        if($db->Execute(buildQuery_Insert($user, SWPHP_DBPREFIX.'users')))
        {
            Msg(SWPHP_Install::l('Se ha creado el administrador').' <b>'.$user['nick'].'</b>', 'check');

            if($data['convertir'])
            {
                $kptluser = array(
                    'user_id'=>$db->Insert_id(),
                    'nombre'=>'',
                    'fechanacimiento'=>'0000-00-00',
                    'fechayhoraalta'=>$ahora,
                    'puntos_show'=>0,
                    'puntos_real'=>0,
                    'puntos_recibidos'=>0,
                    'puntos_emitidos'=>0,
                    );
                if($db->Execute(buildQuery_Insert($kptluser, SWPHP_DBPREFIX.'kptl_usuarios')))
                {
                    Msg(SWPHP_Install::l('Se ha convertido en usuario Kptl a').' <b>'.$user['nick'].'</b>', 'check');
                }
                else
                {
                    Msg(SWPHP_Install::l('No se pudo convertir en usuario Kptl a').' <b>'.$user['nick'].'</b><br />'.$db->Error(), 'error');

                    $ok = false;
                }
            }
        }
        else
        {
            Msg(SWPHP_Install::l('No se pudo crear el administrador').' <b>'.$user['nick'].'</b><br />'.$db->Error(), 'error');

            $ok = false;
        }

        return $ok;
    }
    /**
     * Obtiene una cadena en el idioma actual
     *
     * @param string $str
     * @return string
     */
    public static function l($str, $concepto='install', $isocode=null)
    {
        global $swphp_lang;

        $str = trim($str);
        $hash = md5($concepto.$str);

        $return = $str;

        if(isset($swphp_lang[$hash]))
        {
            $return = $swphp_lang[$hash];
        }
        /*
        else
        {
            $swphp_lang[$hash] = $str;

            if(is_null($isocode))
            {
                global $lang;

                $isocode = $lang;
            }

            if($isocode == 'es')
            {
                $fp = fopen(SWPHP_INSTALL_DIR.'lang/frases.txt', 'a');
                fwrite($fp, $str."\n");
                fclose($fp);
            }
        }
        */

        return $return;
    }
}
?>